const assert = require('assert');
const { When, Then } = require('@cucumber/cucumber');

const { fixtures } = require('../../fixtures/server');

const mockValidConfig = fixtures.get('config-valid');

const xMessageResponse = function (xMessages) {
  const message = fixtures.get('message-vehicle-status');
  let response = [];
  for (let i = 0; i < xMessages; i++) {
    response.push({
      topic: 'broadcast',
      message: JSON.stringify(message),
    });
  }
  return response;
};

When(
  'a mock receive API response is configured to return {int} messages',
  function (xMessages) {
    const response = xMessageResponse(xMessages);
    this.mockAxios.onGet(`${mockValidConfig.api}/receive`).reply(200, response);
  }
);

When(
  'a mock receive API response is configured to return a {int} error',
  function (statusCode) {
    const statusMessages = {
      403: 'Token expired',
      503: 'Service unavailable',
    };
    this.mockAxios
      .onGet(`${mockValidConfig.api}/receive`)
      .reply(statusCode, { message: statusMessages[statusCode] });
  }
);

When('the poll method is called', function () {
  this.call = this.adapter.poll();
  this.callCounts.poll = this.adapter.getTrackedCalls('poll').length;
});

Then(
  'a successful response is returned with {int} messages',
  function (xMessages) {
    this.call.then((response) => {
      assert.equal(response.data.length, xMessages);
    });
  }
);

Then(
  'the protocol {string} method is called {int} times',
  function (method, xInvokes) {
    const decodes = this.protocol.getTrackedCalls(method);
    assert.equal(decodes.length, xInvokes);
  }
);

When('the poll method is called with is_retry on', function () {
  this.call = this.adapter.poll(true);
  this.callCounts.poll = this.adapter.getTrackedCalls('poll').length;
});

Then('the poll method was called with is_retry on', function () {
  let pollCalls = this.adapter.getTrackedCalls('poll');
  let lastCall = pollCalls[pollCalls.length - 1];
  assert.ok(lastCall.arguments[1].is_retry);
});

Then('the poll method is not called again', function () {
  let newPollCallCount = this.adapter.getTrackedCalls('poll').length;
  assert.equal(this.callCounts.poll, newPollCallCount);
});
